import React from "react";

export const useKeyLister = (fn: Function) => {
  // eslint-disable-next-line react-hooks/exhaustive-deps
  const pressed = new Map();

  const registerKeyPress = React.useCallback(
    (event: KeyboardEvent, codes: string[], callBack: Function) => {
      if (codes.hasIncludeElement(event.code)) {
        pressed.addValueOrMakeCallback(event.code, event.type, (e) => {
          if (Array.from(pressed.values()).equals(["keydown", "keydown"], false)) {
            callBack();
          }
        });
      }
    },
    [pressed]
  );

  React.useEffect(() => {
    window.addEventListener("keyup", (e) => registerKeyPress(e, ["KeyQ", "KeyW"], () => fn));
    window.addEventListener("keydown", (e) => registerKeyPress(e, ["KeyQ", "KeyW"], () => {}));
  }, [fn, registerKeyPress]);

  return [];
};
