import { DatasetsScreenPath } from "../../../features/dataset/dataset_screen";
import { Icon } from "../icons/icons";
import { useNavigate } from "react-router-dom";
import { Spin } from "antd";
import { LoadingOutlined } from "@ant-design/icons";
import React from "react";
import { SceneManagerPath } from "../../../features/scene_manager/presentation/scene_manager";
import { AssemblesScreenPath } from "../../../features/assembles/assembles_screen";
import { DetailsScreenPath } from "../../../features/details/details_screen";
import { SimulationScreenPath } from "../../../features/simulations/simulations_screen";
import { EstimateScreenPath } from "../../../features/estimate/estimate_screen";
import { BehaviorTreeBuilderPath } from "../../../features/behavior_tree_builder/presentation/behavior_tree_builder_screen";
export interface IBlockProps {
  name: string;
  isActive: boolean;
  path: string;
  icon?: string;
}
const Block = (props: IBlockProps) => {
  const navigate = useNavigate();

  return (
    <div onClick={() => navigate(props.path)} style={{ height: 56, cursor: "pointer" }}>
      <div
        style={
          props.isActive
            ? {
                textAlignLast: "center",
                height: 32,
                backgroundColor: "rgba(232, 222, 248, 1)",
                marginLeft: 5,
                marginRight: 5,
                alignContent: "center",
                borderRadius: 12,
              }
            : { textAlignLast: "center", alignContent: "center" }
        }
      >
        <Icon type={props.icon ?? ""} />
      </div>
      <div style={{ textAlignLast: "center" }}>{props.name}</div>
    </div>
  );
};
export interface IMainPageProps {
  page: string;
  bodyChildren?: JSX.Element;
  isLoading?: boolean;
}
export const MainPage = (props: IMainPageProps) => {
  const blocksNames = [
    { name: "Детали", path: DetailsScreenPath, icon: "Setting" },
    { name: "Сборки", path: AssemblesScreenPath, icon: "Assembly" },
    { name: "Датасеты", path: DatasetsScreenPath, icon: "Datasets" },
    { name: "Сцена", path: SceneManagerPath, icon: "Layers" },
    { name: "Навыки", path: BehaviorTreeBuilderPath, icon: "Rocket" },
    { name: "Симуляция", path: SimulationScreenPath, icon: "Simulation" },
    { name: "Оценка", path: EstimateScreenPath, icon: "Grade" },
  ];
  const blocks: IBlockProps[] = blocksNames
    .map((el) => {
      return Object.assign({ isActive: false }, el);
    })
    .map((el) => {
      if (el.name.isEqual(props.page)) {
        el.isActive = true;
        return el;
      }
      return el;
    });
  React.useEffect(() => {
    document.body.style.overflow = "hidden";
    return () => {
      document.body.style.overflow = "scroll";
    };
  });
  return (
    <div style={{ display: "flex" }}>
      <div
        style={{
          width: 90,
          height: window.innerHeight,
          display: "flex",
          flexDirection: "column",
          justifyContent: "space-between",
          alignItems: "center",
        }}
      >
        <div style={{ paddingTop: 43 }}>
          <div style={{ textAlignLast: "center" }}>
            <Icon type="MenuFab" />
          </div>
          <div
            style={{
              textAlignLast: "center",
              width: 56,
              height: 56,
              borderRadius: 12,
              backgroundColor: "#ffd9e4",
              alignContent: "center",
            }}
          >
            <Icon style={{ marginTop: 3 }} type="Pencil" />
          </div>
        </div>
        <div>
          {blocks.map((el) => (
            <Block isActive={el.isActive} name={el.name} path={el.path} icon={el.icon} />
          ))}
        </div>
        <div style={{ paddingBottom: 10 }}>
          <Icon type={"Settings"} />
        </div>
      </div>
      {props.isLoading ? (
        <div style={{ alignContent: "center", width: "100%", textAlign: "center" }}>
          <Spin indicator={<LoadingOutlined style={{ fontSize: 68, color: "rgba(103, 80, 164, 1)" }} spin />} />
        </div>
      ) : (
        <>
          <div style={{ width: 241, height: window.innerHeight, backgroundColor: "#F7F2FA", borderRadius: 16 }}> </div>
          {props.bodyChildren}
        </>
      )}
    </div>
  );
};
