import makeAutoObservable from "mobx-store-inheritance";
import { Box3, Vector2, Vector3 } from "three";
import { UiErrorState } from "../../core/store/base_store";
import { HttpError } from "../../core/repository/http_repository";
import { UiBaseError } from "../../core/model/ui_base_error";
import { RobossemblerFiles } from "../scene_manager/model/scene_assets";
import { StickObjectsMarkingThreeRepository } from "./stick_objects_marking_three_repository";
import { UserData } from "../../core/repository/core_three_repository";
import { SceneHttpRepository } from "../scene_manager/data/scene_repository";
import { Asset, RobossemblerAssets } from "../../core/model/robossembler_assets";
import { StickObjectsMarkingStoreMode } from "./model/stick_objects_marking_store_mode";
import { StickyHelper } from "./model/sticky_helper";
import { StickyLoaderMode } from "./model/sticky_loader_mode";

export class StickObjectsMarkingStore extends UiErrorState<HttpError> {
  mode: StickObjectsMarkingStoreMode;
  stickObjectsMarkingThreeRepository: null | StickObjectsMarkingThreeRepository = null;
  sceneHttpRepository: SceneHttpRepository;
  objectThatSticksName: string;
  objectThatSticksNamePoints: Vector3[] = [];
  objectsToWhichItSticksName: string;
  objectsToWhichItSticksPoints: Vector3[] = [];
  sceneAssets?: RobossemblerAssets;
  points = {};

  get stickyObjects() {
    return this.sceneAssets?.assets.filter((el) => el.actions.includes("Sticking"));
  }

  constructor() {
    super();
    makeAutoObservable(this);
    this.points = {};
    this.mode = StickObjectsMarkingStoreMode.move;
    this.sceneHttpRepository = new SceneHttpRepository();
  }
  updatePoint = (key: string, value: any) => {
    // @ts-expect-error
    this.points[key] = value;
  };
  onSaveSticky(): void {}
  setMode(stickObjectsMarkingStoreMode: StickObjectsMarkingStoreMode): void {
    this.mode = stickObjectsMarkingStoreMode;
  }

  loaderWatcher() {}

  async init(): Promise<any> {
    this.mapOk("sceneAssets", this.sceneHttpRepository.getRobossemblerAssets());
  }

  errorHandingStrategy = (error: HttpError) => {
    if (error.status === 404) {
      this.errors.push(new UiBaseError(`${RobossemblerFiles.robossemblerAssets} not found to project`));
    }
  };

  async loadScene(canvasRef: HTMLCanvasElement) {
    this.loadWebGl(canvasRef);
  }
  watcherSceneEditorObject() {}
  loadWebGl(canvasRef: HTMLCanvasElement): void {
    this.stickObjectsMarkingThreeRepository = new StickObjectsMarkingThreeRepository(
      canvasRef as HTMLCanvasElement,
      this.watcherSceneEditorObject,
      this.updatePoint
    );
    this.stickObjectsMarkingThreeRepository.stickyHelperLoader([
      new StickyHelper(
        this.objectThatSticksName,
        this.objectThatSticksNamePoints,
        this.objectsToWhichItSticksName,
        this.objectsToWhichItSticksPoints
      ),
    ]);
    this.stickObjectsMarkingThreeRepository.render();
    window.addEventListener("click", (event) => this.clickLister(event));
  }

  clickLister(event: MouseEvent) {
    const vector = new Vector2();
    vector.x = (event.clientX / window.innerWidth) * 2 - 1;
    vector.y = -(event.clientY / window.innerHeight) * 2 + 1;
    if (this.mode) {
      if (this.mode === StickObjectsMarkingStoreMode.move) {
        this.stickObjectsMarkingThreeRepository?.setRayCastAndGetFirstObject(vector).fold(
          (success) => this.stickObjectsMarkingThreeRepository?.setTransformControlsAttach(success),
          (_error) => this.stickObjectsMarkingThreeRepository?.disposeTransformControlsMode()
        );
      }
      this.stickObjectsMarkingThreeRepository?.setRayCast(vector).map((touch) => {
        const objectMagnetism = touch.filter((el) => el.object.userData[UserData.objectForMagnetism] === true);
        const BoundBoxVector = new Box3().setFromObject(objectMagnetism[0].object).getCenter(new Vector3());
        const centerRelativeVector = new Vector3().subVectors(BoundBoxVector, objectMagnetism[0].point);
        if (objectMagnetism.isNotEmpty()) {
          if (this.mode === StickObjectsMarkingStoreMode.objectsToWhichItSticks) {
            this.objectsToWhichItSticksName = objectMagnetism[0].object.name;
          }
          if (this.mode === StickObjectsMarkingStoreMode.objectThatSticks) {
            this.objectThatSticksName = objectMagnetism[0].object.name;
          }
          if (this.mode === StickObjectsMarkingStoreMode.addPointsObjectThatSticks) {
            this.objectThatSticksNamePoints.push(centerRelativeVector);
          }
          if (this.mode === StickObjectsMarkingStoreMode.addPointsObjectsToWhichItSticks) {
            this.objectsToWhichItSticksPoints.push(centerRelativeVector);
          }
        }
      });
    }
  }
  loadAsset(asset: Asset, mode: StickyLoaderMode): void {
    this.stickObjectsMarkingThreeRepository?.loadInstance(asset,() =>{
      
    });
  }
  dispose() {
    window.removeEventListener("click", this.clickLister);
  }
}
