import { Box3, BoxGeometry, Mesh, MeshBasicMaterial, Object3D, Vector3 } from "three";
import { CoreThreeRepository } from "../../core/repository/core_three_repository";
import { CoreVector3 } from "../../core/model/core_vector3";
import { StickyHelper } from "./model/sticky_helper";

export class StickObjectsMarkingThreeRepository extends CoreThreeRepository {
  stickyObjects: StickyHelper[];
  drawUiPoint: Function;
  constructor(htmlCanvasRef: HTMLCanvasElement, watcherSceneEditorObject: Function, updatePoint: Function) {
    super(htmlCanvasRef, watcherSceneEditorObject);
    this.drawUiPoint = updatePoint;
    this.sceneWatcher();
  }

  getStickyObject(name: string, pointNameHelper: string, index: number) {
    const objectThatSticksNameMesh = this.scene.getObjectByName(name);
    const pointName = objectThatSticksNameMesh!.name + ":point:" + index + pointNameHelper;
    return this.scene.getObjectByName(pointName);
  }

  mapperStickyObject(point: Vector3, index: number, name: string, pointNameHelper: string) {
    const objectThatSticksNameMesh = this.scene.getObjectByName(name);
    const pointName = objectThatSticksNameMesh!.name + ":point:" + index + pointNameHelper;
    const sceneObject = this.scene.getObjectByName(pointName);
    let pointMesh: Object3D;
    if (sceneObject) {
      pointMesh = sceneObject;
    } else {
      pointMesh = new Mesh(new BoxGeometry(1.1, 1.1, 1.1), new MeshBasicMaterial({ color: "#8BC34A" }));
    }
    pointMesh.position.copy(new Box3().setFromObject(objectThatSticksNameMesh!).getCenter(new Vector3()).add(point));
    pointMesh.name = pointName;
    if (sceneObject === undefined) this.scene.add(pointMesh);
  }

  stickyHelperLoader(stickyObjects: StickyHelper[]) {
    this.stickyObjects = stickyObjects;

    stickyObjects.forEach((el) => {
      el.objectThatSticksNamePoints.forEach((point, index) =>
        this.mapperStickyObject(point, index, el.objectThatSticksName, "objectThatSticksNamePoints")
      );
      el.objectsToWhichItSticksPoints.forEach((point, index) =>
        this.mapperStickyObject(point, index, el.objectsToWhichItSticksName, "objectsToWhichItSticksPoints")
      );
    });
  }
  getCenter = (obj: Object3D) => new Box3().setFromObject(obj).getCenter(new Vector3());

  sceneWatcher() {
    this.transformControls.addEventListener("objectChange", (event) => {
      //@ts-expect-error
      const sceneActiveObject = event.target.object as Mesh;

      this.stickyObjects.forEach((stickyHelper) => {
        if (sceneActiveObject.name === stickyHelper.objectThatSticksName) {
          const objectsToWhichItSticksPointLocalVector = stickyHelper.objectsToWhichItSticksPoints[0];
          const globalVectorObjStickyName = this.scene.getObjectByName(stickyHelper.objectThatSticksName);

          const globalVectorObjToWhichSticks = this.scene.getObjectByName(stickyHelper.objectsToWhichItSticksName);

          const objectsToWhichItSticksNamePosition = new CoreVector3(globalVectorObjToWhichSticks!.position).add(
            objectsToWhichItSticksPointLocalVector
          ).vector;

          this.scene
            .getObjectByName("cube2:point:0objectThatSticksNamePoints")
            ?.position.copy(objectsToWhichItSticksNamePosition);

          globalVectorObjStickyName?.position.copy(
            this.scene
              .getObjectByName("cube2:point:0objectThatSticksNamePoints")!
              .position.add(objectsToWhichItSticksPointLocalVector)
          );
        }
      });
    });
  }
}
