import makeAutoObservable from "mobx-store-inheritance";
import { ProjectRepository, UUID } from "../data/project_repository";
import { IProjectModel } from "../model/project_model";
import { SimpleErrorState } from "../../../core/store/base_store";
import { ActivePipeline } from "../../../core/model/active_pipeline";

interface IProjectView {
  isActive: boolean;
  description: string;
  id: string;
}
export class ProjectView {
  isActive: boolean;
  description: string;
  id: string;
  constructor(view: IProjectView) {
    this.isActive = view.isActive;
    this.description = view.description;
    this.id = view.id;
  }
}
export class AllProjectStore extends SimpleErrorState {
  projectsModels?: IProjectModel[];
  activeProjectId?: UUID;
  repository: ProjectRepository;
  constructor(repository: ProjectRepository) {
    super();
    this.repository = repository;
    makeAutoObservable(this);
  }
  async getProjects(): Promise<void> {
    await this.mapOk<IProjectModel[]>("projectsModels", this.repository.getAllProject());
  }
  async getActiveProjectId(): Promise<void> {
    await this.mapOk<UUID>("activeProjectId", this.repository.getActivePipeline());
  }
  async init() {
    await Promise.all([this.getProjects(), this.getActiveProjectId()]);
    this.projectsModels?.map((el) => (el._id === this.activeProjectId ? ((el.isActive = true), el) : el));
  }

  async setPipelineActive(id: string) {
    await this.httpHelper(this.repository.setActivePipeline(id));
  }
}
