import { TypedEvent } from "../../../core/helper/typed_event";
import { NodeEditor } from "rete";
import { AreaExtra, Schemes } from "../presentation/ui/editor/editor";
import { Result } from "../../../core/helper/result";
import { AreaPlugin } from "rete-area-plugin";

export interface BtDrawDragAndDropView {
  x: number;
  y: number;
  name: string;
}

export class BtNodeView extends TypedEvent<BtDrawDragAndDropView> {}

export class ReteObserver extends TypedEvent<any> {}
export class BtBuilderModel {
  public static result = "";
  static fromReteScene(editor: NodeEditor<Schemes>, area: AreaPlugin<Schemes, AreaExtra>): Result<string, string> {
    try {
      this.getFirstSequence(editor).map((sortedSequence) => {
        const firstNodeId = sortedSequence.getKeyFromValueIsExists(1) as string;
        this.findSequence(firstNodeId, editor, sortedSequence, 2);
        this.result += `<${this.getNodeLabelAtId(editor, firstNodeId)} id="${firstNodeId}">`;
        this.toXML(sortedSequence as Map<string, number>, editor, area, firstNodeId);
        this.result += `</${this.getNodeLabelAtId(editor, firstNodeId)}>`;
      });
      return Result.ok(this.result);
    } catch (error) {
      return Result.error("BtBuilderModel fromReteScene error");
    }
  }
  public static getNodeLabelAtId(editor: NodeEditor<Schemes>, id: string) {
    return editor.getNode(id).label;
  }

  static toXML(
    sortedSequence: Map<string, number>,
    editor: NodeEditor<Schemes>,
    area: AreaPlugin<Schemes, AreaExtra>,
    activeElementId = ""
  ) {
    editor.getConnections().forEach((el) => {
      if (el.source === activeElementId) {
        this.result += `<${this.getNodeLabelAtId(editor, el.target)} id="${el.target}">`;
        this.toXML(sortedSequence, editor, area, el.target);
        this.result += `</${this.getNodeLabelAtId(editor, el.target)}>`;
      }
    });
  }

  public static getBtPriorities(ids: string[]) {}
  public static findSequence(
    nodeId: string,
    editor: NodeEditor<Schemes>,
    result: Map<string, boolean | number>,
    lvl: number
  ): Map<string, number> | undefined {
    const connections: string[] = [];

    editor.getConnections().forEach((el) => {
      if (el.source === nodeId) {
        connections.push(el.target);
      }
    });

    if (connections.isEmpty()) return result as Map<string, number>;
    let lv = lvl;
    connections.forEach((el) => {
      result.set(el, lvl);
      this.findSequence(el, editor, result, (lv += 1));
    });
  }
  public static getFirstSequence(editor: NodeEditor<Schemes>): Result<undefined, Map<string, boolean | number>> {
    const node = new Map<string, boolean | number>();
    editor.getNodes().forEach((el) => {
      node.set(el.id, 1);
    });

    editor.getConnections().forEach((el) => {
      node.set(el.target, false);
    });

    const key = node.getKeyFromValueIsExists(1);
    if (key === undefined) {
      return Result.error(undefined);
    }
    return Result.ok(node);
  }
}
