import makeAutoObservable from "mobx-store-inheritance";
import xmlFormat from "xml-formatter";
import { HttpError } from "../../../core/repository/http_repository";
import { UiErrorState } from "../../../core/store/base_store";
import { BtBuilderModel, BtNodeView, ReteObserver } from "../model/editor_view";
import { NodeEditor } from "rete";
import { AreaExtra, Schemes } from "./ui/editor/editor";
import { AreaPlugin } from "rete-area-plugin";
import { NodeBehaviorTree } from "../model/node_behavior_tree";

interface I2DArea {
  x: number;
  y: number;
  w: number;
  h: number;
}

export class BehaviorTreeBuilderStore extends UiErrorState<HttpError> {
  saveBt(): void {
    this.reteNode?.emit("200");
  }
  validateBt() {}
  area?: I2DArea;
  btNodeView: BtNodeView = new BtNodeView();
  reteNode?: ReteObserver;
  canRun = true;
  nodes: NodeBehaviorTree[] = [
    {
      label: "Sequence",
      id: "1",
      outputs: ["a"],
      inputs: ["a"],
      position: { x: -488.1303402823156, y: -227.90861570911895 },
    },
    {
      label: "Sequence",
      id: "2",
      outputs: ["a"],
      inputs: ["a"],
      connectTo: "1",
      position: { x: 119.95735514023244, y: -182.24902817216878 },
    },
    {
      label: "Sequence",
      id: "6",
      outputs: ["a"],
      inputs: ["a"],
      connectTo: "2",
      position: { x: 402.06111561958505, y: -100.97814086372247 },
    },
    {
      label: "MoveToPose",
      id: "7",
      outputs: ["a"],
      inputs: ["a"],
      connectTo: "6",
      position: { x: 932.0688448287292, y: 90.10780461923443 },
    },
    {
      label: "MoveToPose",
      id: "8",
      outputs: ["a"],
      inputs: ["a"],
      connectTo: "6",
      position: { x: 898.1684213039546, y: -119.80545806921208 },
    },
    {
      label: "MoveToPose",
      id: "3",
      outputs: [],
      inputs: ["a"],
      connectTo: "1",
      position: { x: 208.65749743442188, y: 16.256489050033785 },
    },
    {
      label: "MoveToPose",
      id: "4",
      outputs: [],
      inputs: ["a"],
      connectTo: "1",
      position: { x: -50.46426323140673, y: 99.22642447650014 },
    },
    {
      label: "Sequence",
      id: "5",
      outputs: ["a"],
      inputs: ["a"],
      connectTo: "1",
      position: { x: -234.54554662642457, y: 245.4012710608967 },
    },
  ];

  constructor() {
    super();
    makeAutoObservable(this);
  }

  setBt(value: string): void {
    JSON.parse(value) as BtNodeView[];
  }
  bt = async (editor: NodeEditor<Schemes>, area: AreaPlugin<Schemes, AreaExtra>) => {
    (await BtBuilderModel.fromReteScene(editor, area)).fold(
      (s) => {
        console.log(
          xmlFormat(`<?xml version="1.0" encoding="UTF-8"?>
        <root main_tree_to_execute="Main">
            <BehaviorTree ID="Main">
                ${s}
            </BehaviorTree>
        
            <TreeNodesModel>
                <Action ID="RbsBtAction">
                    <input_port name="do" />
                    <input_port name="command" />
                    <input_port name="server_name" />
                    <input_port name="server_timeout" />
                </Action>
            </TreeNodesModel>
        
        </root>`)
        );
      },
      (_) => _
    );
  };
  copyBt = () => {
    this.reteNode?.emit("200");
  };
  errorHandingStrategy: (error: HttpError) => void;

  dragEnd = (e: EventTarget) => {
    if (this.canRun) {
      //@ts-expect-error
      this.drawSkillCheck(e.x as number, e.y as number, e.target.innerText as string);
      this.canRun = false;
      setTimeout(() => {
        this.canRun = true;
      }, 100);
    }
  };
  drawSkillCheck(x: number, y: number, name: string) {
    const drawPoint = { x: x, y: y, w: 1, h: 1 };

    if (
      drawPoint.x < this.area!.x + this.area!.w &&
      drawPoint.x + drawPoint.w > this.area!.x &&
      drawPoint.y < this.area!.y + this.area!.h &&
      drawPoint.y + drawPoint.h > this.area!.y
    ) {
      this.btNodeView.emit({
        x: x,
        y: y - (this.area!.y + this.area!.h / 2),
        name: name,
      });
    }
  }

  async init(): Promise<any> {}
  dragZoneSetOffset(offsetTop: number, offsetWidth: number, width: number, height: number) {
    this.area = {
      x: offsetTop,
      y: offsetWidth,
      w: width,
      h: height,
    };
  }
  dispose() {}
}
