import * as React from "react";
import TreeView, { EventCallback, IBranchProps, INode, LeafProps, flattenTree } from "react-accessible-treeview";
import { IFlatMetadata } from "react-accessible-treeview/dist/TreeView/utils";
import { CallBackEventTarget } from "../../../../../core/extensions/extensions";
export interface ISkillTreeProps {
  skills: any;
  dragEnd: CallBackEventTarget;
}

interface IRefListerProps {
  element: INode<IFlatMetadata>;
  getNodeProps: (args?: { onClick?: EventCallback }) => IBranchProps | LeafProps;
  handleExpand: EventCallback;
  handleSelect: EventCallback;
  dragEnd: CallBackEventTarget;
}

export const RefListener = (props: IRefListerProps) => {
  const ref = React.useRef<HTMLDataElement>(null);
  React.useEffect(() => {
    if (ref.current) {
      ref.current.addEventListener("dragend", (e) => {
        // @ts-expect-error
        if (e.target.innerHTML) {
          // @ts-expect-error
          props.dragEnd(e);
        }
      });
    }
  }, [ref, props]);

  return (
    <div {...props.getNodeProps({ onClick: props.handleExpand })}>
      <div
        onClick={(e) => {
          props.handleSelect(e);
        }}
      />
      <span ref={ref} style={{ color: "white" }} draggable="true">
        {props.element.name}
      </span>
    </div>
  );
};
export function SkillTree(props: ISkillTreeProps) {
  return (
    <div>
      <TreeView
        data={flattenTree(props.skills)}
        aria-label="onSelect"
        onSelect={() => {}}
        multiSelect
        defaultExpandedIds={[1, 2]}
        nodeAction="check"
        nodeRenderer={({ element, getNodeProps, handleSelect, handleExpand }) => {
          return (
            <RefListener
              dragEnd={props.dragEnd}
              getNodeProps={getNodeProps}
              element={element}
              handleExpand={handleExpand}
              handleSelect={handleSelect}
            />
          );
        }}
      />
    </div>
  );
}
