import { Result } from "../../core/helper/result";
import { HttpError, HttpMethod, HttpRepository } from "../../core/repository/http_repository";
import { UUID } from "../all_projects/data/project_repository";
import { Assets, DataSetModel, Dataset, IDatasetModel, ProcessStatus } from "./dataset_model";

export class DataSetRepository extends HttpRepository {
  editDataset(dataSetModel: DataSetModel) {
    dataSetModel.processStatus = ProcessStatus.NEW;
    return this._jsonRequest<void>(HttpMethod.PUT, `/datasets`, dataSetModel);
  }
  deleteDataset(id: string) {
    return this._jsonRequest<void>(HttpMethod.DELETE, `/datasets/dataset?id=${id}`);
  }
  getActiveProjectId(): Promise<Result<HttpError, UUID>> {
    return this._jsonRequest<UUID>(HttpMethod.GET, "/projects/get/active/project/id");
  }
  getAssetsActiveProject = async (): Promise<Result<HttpError, Assets>> => {
    return this._jsonRequest<Assets>(HttpMethod.GET, "/projects/assets");
  };
  saveDataSet = async (model: DataSetModel) => {
    return this._jsonRequest<Dataset>(HttpMethod.POST, "/datasets", model);
  };
  getDatasetsActiveProject = async (): Promise<Result<HttpError, IDatasetModel[]>> => {
    return this._jsonRequest(HttpMethod.GET, "/datasets");
  };
  execDatasetProcess = async (id: string) => {
    return this._jsonRequest(HttpMethod.POST, `/datasets/exec?id=${id}`);
  };
  isRunningProcess = async () => {
    return this._jsonRequest(HttpMethod.GET, "/datasets/is/running");
  };
  deleteProcess = async () => {
    return this._jsonRequest(HttpMethod.GET, "/datasets/delete/process");
  };
}
