import * as React from "react";
import { observer } from "mobx-react-lite";
import { useParams } from "react-router-dom";
import { Button } from "antd";
import { StickObjectsMarkingStore, StickObjectsMarkingStoreMode } from "./stick_objects_marking_store";
import { Vector3 } from "three";

export const StickObjectsMarking = "/stick/objects/marking";
interface StickButtonsProps {
  isVisible: boolean;
  name: string;
  groupMode: StickObjectsMarkingStoreMode;
  storeMode: StickObjectsMarkingStoreMode;
  storeModePoints: StickObjectsMarkingStoreMode;
  setMode: Function;
  setPointMode: Function;
  points: Vector3[];
  body: string;
}

export const StickButtons: React.FunctionComponent<StickButtonsProps> = observer((props) => {
  return (
    <>
      {props.isVisible ? (
        <>
          {props.name}
          <Button
            style={{
              backgroundColor: props.storeMode === props.storeModePoints ? "#ff7e1e" : "",
            }}
            onClick={() => props.setPointMode()}
          >
            add points
          </Button>
          {props.points.map((el) => {
            return (
              <>
                {el.x} {el.y} {el.z}
              </>
            );
          })}
        </>
      ) : (
        <Button
          style={{
            backgroundColor: props.groupMode === props.storeMode ? "#ff7e1e" : "",
          }}
          onClick={() => props.setMode(props.storeMode)}
        >
          {props.body}
        </Button>
      )}
    </>
  );
});

export const StickObjectsMarkingScreen = observer(() => {
  const canvasRef = React.useRef<HTMLCanvasElement>(null);
  const [store] = React.useState(() => new StickObjectsMarkingStore());
  const id = useParams().id as string;

  React.useEffect(() => {
    store.init();
    store.loadScene(canvasRef.current!);
    document.body.style.overflow = "hidden";
    return () => {
      document.body.style.overflow = "scroll";
      store.dispose();
    };
  }, [id, store]);

  return (
    <div>
      <canvas ref={canvasRef} style={{ position: "absolute", overflow: "hidden" }} />
      <div
        style={{
          display: "flex",
          flexDirection: "row",
          alignContent: "center",
          justifyContent: "space-between",
          position: "absolute",
          width: "100vw",
        }}
      >
        <div style={{ backgroundColor: "white", padding: "20px" }}>
          {Object.keys(store.points).map((el) => {
            // @ts-expect-error
            const v = store.points[el];
            return (
              <>
                <div>
                  {el as string}:{v}
                </div>
              </>
            );
          })}
          <div>Marking objects for sticking</div>
          <div>
            <StickButtons
              storeMode={store.mode}
              setMode={() => store.setMode(StickObjectsMarkingStoreMode.objectThatSticks)}
              groupMode={StickObjectsMarkingStoreMode.objectThatSticks}
              name={store.objectThatSticksName}
              storeModePoints={StickObjectsMarkingStoreMode.addPointsObjectThatSticks}
              isVisible={store.objectThatSticksName !== undefined}
              setPointMode={() => store.setMode(StickObjectsMarkingStoreMode.addPointsObjectThatSticks)}
              points={store.objectThatSticksNamePoints}
              body="objectThatSticksName"
            />
            <StickButtons
              body="objectsToWhichItSticksName"
              points={store.objectsToWhichItSticksPoints}
              storeMode={store.mode}
              setMode={() => store.setMode(StickObjectsMarkingStoreMode.objectsToWhichItSticks)}
              groupMode={StickObjectsMarkingStoreMode.objectsToWhichItSticks}
              name={store.objectsToWhichItSticksName}
              storeModePoints={StickObjectsMarkingStoreMode.addPointsObjectsToWhichItSticks}
              isVisible={store.objectsToWhichItSticksName !== undefined}
              setPointMode={() => store.setMode(StickObjectsMarkingStoreMode.addPointsObjectsToWhichItSticks)}
            />
            <Button onClick={() => store.onSaveSticky()}>save</Button>
            <Button
              onClick={() => store.setMode(StickObjectsMarkingStoreMode.move)}
              style={{ backgroundColor: store.mode === StickObjectsMarkingStoreMode.move ? "#ff7e1e" : "" }}
            >
              run
            </Button>
          </div>
        </div>
      </div>
    </div>
  );
});
