import makeAutoObservable from "mobx-store-inheritance";
import { Box3, Vector2, Vector3 } from "three";
import { UiErrorState } from "../../core/store/base_store";
import { HttpError } from "../../core/repository/http_repository";
import { UiBaseError } from "../../core/model/ui_base_error";
import { RobossemblerFiles } from "../scene_manager/model/scene_assets";
import { StickObjectsMarkingThreeRepository } from "./stick_objects_marking_three_repository";
import { UserData } from "../../core/repository/core_three_repository";
import { ObjectExtensionsIsKeyExists } from "../../core/extensions/object";

export enum StickObjectsMarkingStoreMode {
  objectsToWhichItSticks = "objectsToWhichItSticks",
  objectThatSticks = "objectThatSticks",
  addPointsObjectsToWhichItSticks = "addPointsObjectsToWhichItSticks",
  addPointsObjectThatSticks = "addPointsObjectThatSticks",
  move = "move",
}

export class StickyHelper {
  objectThatSticksName: string;
  objectThatSticksNamePoints: Vector3[] = [];
  objectsToWhichItSticksName: string;
  objectsToWhichItSticksPoints: Vector3[] = [];

  constructor(
    objectThatSticksName: string,
    objectThatSticksNamePoints: Vector3[],
    objectsToWhichItSticksName: string,
    objectsToWhichItSticksPoints: Vector3[]
  ) {
    this.objectThatSticksName = objectThatSticksName;
    this.objectThatSticksNamePoints = objectThatSticksNamePoints;
    this.objectsToWhichItSticksName = objectsToWhichItSticksName;
    this.objectsToWhichItSticksPoints = objectsToWhichItSticksPoints;
  }
}

export class StickObjectsMarkingStore extends UiErrorState<HttpError> {
  mode: StickObjectsMarkingStoreMode;
  stickObjectsMarkingThreeRepository: null | StickObjectsMarkingThreeRepository = null;
  objectThatSticksName: string;
  objectThatSticksNamePoints: Vector3[] = [];
  objectsToWhichItSticksName: string;
  objectsToWhichItSticksPoints: Vector3[] = [];
  points = {};
  constructor() {
    super();
    makeAutoObservable(this);
    this.points = {};
    this.mode = StickObjectsMarkingStoreMode.move;
    this.loadStickyJSON(
      '{"objectThatSticksName":"cube2","objectThatSticksNamePoints":[{"x":5,"y":-0.5419443937360455,"z":0.041458499858311626}],"objectsToWhichItSticksName":"cube1","objectsToWhichItSticksPoints":[{"x":-5,"y":-2.2134708060033113,"z":1.1937718220731925}]}'
    );
  }
  updatePoint = (key: string, value: any) => {
    // @ts-expect-error
    this.points[key] = value;
  };
  onSaveSticky(): void {
    window.prompt(
      "Copy to clipboard: Ctrl+C, Enter",
      JSON.stringify(
        new StickyHelper(
          this.objectThatSticksName,
          this.objectThatSticksNamePoints,
          this.objectsToWhichItSticksName,
          this.objectsToWhichItSticksPoints
        )
      )
    );
  }
  setMode(stickObjectsMarkingStoreMode: StickObjectsMarkingStoreMode): void {
    this.mode = stickObjectsMarkingStoreMode;
  }

  loaderWatcher() {}

  async init(): Promise<any> {}

  errorHandingStrategy = (error: HttpError) => {
    if (error.status === 404) {
      this.errors.push(new UiBaseError(`${RobossemblerFiles.robossemblerAssets} not found to project`));
    }
  };

  async loadScene(canvasRef: HTMLCanvasElement) {
    this.loadWebGl(canvasRef);
  }
  watcherSceneEditorObject() {}
  loadWebGl(canvasRef: HTMLCanvasElement): void {
    this.stickObjectsMarkingThreeRepository = new StickObjectsMarkingThreeRepository(
      canvasRef as HTMLCanvasElement,
      this.watcherSceneEditorObject,
      this.updatePoint
    );
    this.stickObjectsMarkingThreeRepository.stickyHelperLoader([
      new StickyHelper(
        this.objectThatSticksName,
        this.objectThatSticksNamePoints,
        this.objectsToWhichItSticksName,
        this.objectsToWhichItSticksPoints
      ),
    ]);
    this.stickObjectsMarkingThreeRepository.render();
    window.addEventListener("click", (event) => this.clickLister(event));
  }

  clickLister(event: MouseEvent) {
    const vector = new Vector2();
    vector.x = (event.clientX / window.innerWidth) * 2 - 1;
    vector.y = -(event.clientY / window.innerHeight) * 2 + 1;
    if (this.mode) {
      if (this.mode === StickObjectsMarkingStoreMode.move) {
        this.stickObjectsMarkingThreeRepository?.setRayCastAndGetFirstObject(vector).fold(
          (success) => this.stickObjectsMarkingThreeRepository?.setTransformControlsAttach(success),
          (_error) => this.stickObjectsMarkingThreeRepository?.disposeTransformControlsMode()
        );
      }
      this.stickObjectsMarkingThreeRepository?.setRayCast(vector).map((touch) => {
        const objectMagnetism = touch.filter((el) => el.object.userData[UserData.objectForMagnetism] === true);
        const BoundBoxVector = new Box3().setFromObject(objectMagnetism[0].object).getCenter(new Vector3());
        const centerRelativeVector = new Vector3().subVectors(BoundBoxVector, objectMagnetism[0].point);
        if (objectMagnetism.isNotEmpty()) {
          if (this.mode === StickObjectsMarkingStoreMode.objectsToWhichItSticks) {
            this.objectsToWhichItSticksName = objectMagnetism[0].object.name;
          }
          if (this.mode === StickObjectsMarkingStoreMode.objectThatSticks) {
            this.objectThatSticksName = objectMagnetism[0].object.name;
          }
          if (this.mode === StickObjectsMarkingStoreMode.addPointsObjectThatSticks) {
            this.objectThatSticksNamePoints.push(centerRelativeVector);
          }
          if (this.mode === StickObjectsMarkingStoreMode.addPointsObjectsToWhichItSticks) {
            this.objectsToWhichItSticksPoints.push(centerRelativeVector);
          }
        }
      });
    }
  }

  dispose() {
    window.removeEventListener("click", this.clickLister);
  }
  loadStickyJSON(value: string): void {
    const object = JSON.parse(value) as Object;
    if (
      ObjectExtensionsIsKeyExists(object, [
        "objectThatSticksName",
        "objectThatSticksNamePoints",
        "objectsToWhichItSticksName",
        "objectsToWhichItSticksPoints",
      ])
    ) {
      // @ts-expect-error
      this.objectThatSticksName = object["objectThatSticksName"];
      // @ts-expect-error
      this.objectThatSticksNamePoints = object["objectThatSticksNamePoints"];
      // @ts-expect-error
      this.objectsToWhichItSticksName = object["objectsToWhichItSticksName"];
      // @ts-expect-error
      this.objectsToWhichItSticksPoints = object["objectsToWhichItSticksPoints"];
    }
  }
}
